// Copyright (c) 2014-2019 Sombusta, All Rights Reserved.
// MIT LICENSE // Epic VR Template Convert C++ Open Source Project.

#pragma once
#include "CoreMinimal.h"
#include "Runtime/UMG/Public/Components/WidgetInteractionComponent.h"
#include "GameFramework/Actor.h"
#include "TP_Variables.h"
#include "TP_MotionController.generated.h"

class AWidgetInteractionParent;

UCLASS()
class GLASSGOESINGREEN_API ATP_MotionController : public AActor
{
	GENERATED_BODY()
	
	/// Components Initialization
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "VR|Components", meta = (AllowPrivateAccess = "true"))
		class USceneComponent* RootScene;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "VR|Components", meta = (AllowPrivateAccess = "true"))
		class UMotionControllerComponent* MotionController;
															  
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "VR|Components", meta = (AllowPrivateAccess = "true"))
		class USkeletalMeshComponent* HandMesh;	  				  
															  
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "VR|Components", meta = (AllowPrivateAccess = "true"))
		class USphereComponent* GrabShpere; 					  			  
															  
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "VR|Components", meta = (AllowPrivateAccess = "true"))
		class USteamVRChaperoneComponent* SteamVRChaperone;	


	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "VR|Components", meta = (AllowPrivateAccess = "true"))
		class UStaticMeshComponent* MotionControllerMesh;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "VR|Components", meta = (AllowPrivateAccess = "true"))
		UWidgetInteractionComponent* HandWidgetInteraction;

	

	
public:	
	// Sets default values for this actor's properties
	ATP_MotionController();

	virtual void OnConstruction(const FTransform & Transform) override;

	// Called when the game starts or when spawned
	virtual void BeginPlay() override;

	// Called every frame
	virtual void Tick(float DeltaTime) override;
	
	UFUNCTION(BlueprintCallable, Category = "VR|Grabbing")
		AActor* GetActorNearHand() const;		 
											 
	UFUNCTION(BlueprintCallable, Category = "VR|Grabbing")
		void GrabActor();						 
											 
	UFUNCTION(BlueprintCallable, Category = "VR|Grabbing")
		void ReleaseActor();			 

	UFUNCTION(BlueprintCallable, Category = "VR")
		void RumbleController(float Intensity);

	UFUNCTION()
		void OnComponentBeginOverlap(class UPrimitiveComponent* OverlappedComp, AActor* OtherActor, class UPrimitiveComponent* OtherComp, int32 OtherBodyIndex, bool bFromSweep, const FHitResult& SweepResult);

	UFUNCTION(BlueprintCallable, Category = "VR")
		void UpdateHandAnimation();

	UFUNCTION(BlueprintCallable, Category = "VR")
		bool GetIsRightHand();

	void TriggerStart();
	void TriggerEnd();

private: // Variables Initialization //

	//This is the VR pointer so you can click things. It does nothing if you change it after begin play.
	TSubclassOf<AActor> WidgetInteractionBlueprint = nullptr;

	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "VR|Variables", meta = (AllowPrivateAccess = "true"))
		EGrip_Code CurrentGripState;

	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "VR|Variables", meta = (AllowPrivateAccess = "true"))
		AActor* AttachedActor;

	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "VR|Variables", meta = (AllowPrivateAccess = "true"))
		bool bIsRoomScale;

	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "VR|Variables", meta = (AllowPrivateAccess = "true"))
		bool bWantsToGrip;

	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "VR|Variables", meta = (AllowPrivateAccess = "true"))
		FRotator InitialControllerRotation;

	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "VR|Variables", meta = (AllowPrivateAccess = "true"))
		class UHapticFeedbackEffect_Base* VRHapticEffect;

	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "VR|Variables", meta = (AllowPrivateAccess = "true"))
		class UStaticMesh* BeamMesh;

	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "VR|Variables", meta = (AllowPrivateAccess = "true"))
		class UMaterial* BeamMaterial;


	void GetMotionControllerMeshes();
	void CreateMotionController(bool IsRightHand);

	UStaticMesh* LeftControllerMesh=nullptr;
	UStaticMesh* RightControllerMesh=nullptr;

	UWidgetInteractionComponent* WidgetInteractionComponent=nullptr;
	
	//This is so the pointer knows where the other controller is if it needs to go there.
	UStaticMeshComponent* OtherController = nullptr;

	void SpawnWidget();
public:	


	//Left or right hand enumerator
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "VR|Variables")
		EControllerHand Hand;
	
	FORCEINLINE class UMotionControllerComponent* GetMotionController() const { return MotionController; }
	FORCEINLINE FRotator GetInitialControllerRotation() const { return InitialControllerRotation; }
	void SetOtherController(UStaticMeshComponent* OtherControllerIn);

	UFUNCTION(BlueprintCallable, Category = "VR|Position")
		UStaticMeshComponent* GetOtherController();

	UStaticMeshComponent* GetMotionControllerMesh();
	
	void SetWidgetInteractionBlueprint(TSubclassOf<AActor> NewWidgetInteractionBlueprint);

	void EnableMenu();
	void DisableMenu();
};