// Copyright (c) 2014-2019 Sombusta, All Rights Reserved.
// MIT LICENSE // Epic VR Template Convert C++ Open Source Project.

#pragma once

#include "CoreMinimal.h"
#include "Components/SceneComponent.h"
#include "GameFramework/Pawn.h"
#include "Components/StaticMeshComponent.h"
#include "TP_VirtualRealityPawn_Motion.generated.h"

UCLASS()
class GLASSGOESINGREEN_API ATP_VirtualRealityPawn_Motion : public APawn
{
	GENERATED_BODY()

	///Components
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "PlayerPawn|Components", meta = (AllowPrivateAccess = "true"))
		class USceneComponent* RootScene;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "PlayerPawn|Components", meta = (AllowPrivateAccess = "true"))
		class USceneComponent* CameraBase;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "PlayerPawn|Components", meta = (AllowPrivateAccess = "true"))
		class UCameraComponent* VRCamera;

	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category = "PlayerPawn|MyComponents", meta = (AllowPrivateAccess = "true"))
		class UStaticMeshComponent* PlayerCollision;

	//The minimum Y (or across) value that the player's head can be before it triggers the alarm
	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category = "PlayerPawn|Variables", meta = (AllowPrivateAccess = "true"))
		float MinimumY=-20;

	
		
public:
	// Score keeper currently sets this value.
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "PlayerPawn|Variables", meta = (AllowPrivateAccess = "true"))
		bool GameEnded = false;


	//This is the pointy thing in VR to allow you to click things. Do not change after the motion controllers have been spawned.
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "PlayerPawn|Variables", meta = (AllowPrivateAccess = "true"))
		TSubclassOf<AActor> WidgetInteractionBlueprint = nullptr;

	// Sets default values for this pawn's properties
	ATP_VirtualRealityPawn_Motion();

	// Called when the game starts or when spawned
	virtual void BeginPlay() override;

	// Called every frame
	virtual void Tick(float DeltaTime) override;

	// Called to bind functionality to input
	virtual void SetupPlayerInputComponent(class UInputComponent* PlayerInputComponent) override;
	 
	// Resets HMD orientation and position in VR
	UFUNCTION(BlueprintCallable, Category="VRPawn|Functions")
		void ResetingVR();

	//Levels an actor out to the offset the other actors are at due to pressing and holding the thubmstick
	UFUNCTION(BlueprintCallable, Category = "VRPawn|Functions")
		void OffsetComponentByChangedHeight(USceneComponent* Component);

	///These functions handle grabbing and releasing
	void GrabActor_Left();

	void ReleaseActor_Left();

	void GrabActor_Right();

	void ReleaseActor_Right();


	//Is the player standing in a good place (not on the conveyer belt)
	UFUNCTION(BlueprintCallable,Category="VRPawn|GetVariables")
		bool IsInGoodPosition();

	//These are the actors that move when you press and hold the thumbstick
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "VRPawn|Variables")
		TArray<AActor*> MovableActors = TArray<AActor*>();

	//This is how long you have to hold the thumbstick in seconds for the world to recenter. 
	UPROPERTY(EditAnywhere, BlueprintReadWrite, Category = "VRPawn|Variables", meta = (AllowPrivateAccess = "true"))
		float MinimumTimeForHoldToResetVR = .5f;

	//Whether the player has re-centered their view by pressing and holding the thumbstick. Used for the instructions.
	UPROPERTY(VisibleAnywhere, BlueprintReadWrite, Category = "VRPawn|Instructions", meta = (AllowPrivateAccess = "true"))
		bool HasResetVR = false;
	
	
private:
	//Reference to the instances of the class
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "VR|Variables", meta = (AllowPrivateAccess = "true"))
		class ATP_MotionController* LeftController;

	//Reference to the instances of the class
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "VR|Variables", meta = (AllowPrivateAccess = "true"))
		class ATP_MotionController* RightController;


	UPROPERTY(EditAnywhere, BlueprintReadOnly, Category = "VR|Variables", meta = (AllowPrivateAccess = "true"))
		float DefaultPlayerHeight;

	// Handle to manage the timer of the re-center
	FTimerHandle ResetVRTimerHandle;

	void StartTimerOfResettingVR();

	void StopTimerOfResettingVR();

	bool WorldType__InGameInstanceWorld(UObject* WorldContextObject);

	//Weather the player is standing in a good place (not on the conveyer belt)
	bool bIsInGoodPosition=true;

	///Values to do with re-centering the world
	float SceneMovement=0;

	bool bHasChangedHeight = false;
	void ChangeSceneHeight();
	bool HasGrabbedLeft = false;
	bool HasGrabbedRight = false;

	int ticked = 0;

public:	
	UFUNCTION(BlueprintCallable,Category="Getr")
		ATP_MotionController* GetRightController();

	UFUNCTION(BlueprintCallable, Category = "Getr")
		ATP_MotionController* GetLeftController();
};
